@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1140 $
- * $Date: 2014-10-10 23:13:02 +0200 (Fri, 10 Oct 2014) $
+ * $Revision: 1253 $
+ * $Date: 2015-04-03 07:34:33 -0400 (Fri, 03 Apr 2015) $
  *
  */
 
@@ -158,13 +158,31 @@ void ArticleDownloader::Run()
 			m_pConnection->GetNewsServer()->GetName(), m_pConnection->GetHost());
 		m_szConnectionName[sizeof(m_szConnectionName) - 1] = '\0';
 
+		// check server retention
+		bool bRetentionFailure = m_pConnection->GetNewsServer()->GetRetention() > 0 &&
+			(time(NULL) - m_pFileInfo->GetTime()) / 86400 > m_pConnection->GetNewsServer()->GetRetention();
+		if (bRetentionFailure)
+		{
+			detail("Article %s @ %s failed: out of server retention (file age: %i, configured retention: %i)",
+				m_szInfoName, m_szConnectionName,
+				(time(NULL) - m_pFileInfo->GetTime()) / 86400,
+				m_pConnection->GetNewsServer()->GetRetention());
+			Status = adFailed;
+			FreeConnection(true);
+		}
+
+		if (m_pConnection && !IsStopped())
+		{
+			detail("Downloading %s @ %s", m_szInfoName, m_szConnectionName);
+		}
+
 		// test connection
 		bool bConnected = m_pConnection && m_pConnection->Connect();
 		if (bConnected && !IsStopped())
 		{
 			NewsServer* pNewsServer = m_pConnection->GetNewsServer();
-			detail("Downloading %s @ %s", m_szInfoName, m_szConnectionName);
 
+			// Download article
 			Status = Download();
 
 			if (Status == adFinished || Status == adFailed || Status == adNotFound || Status == adCrcError)
@@ -173,66 +191,45 @@ void ArticleDownloader::Run()
 			}
 		}
 
-		if (bConnected)
-		{
-			if (Status == adConnectError)
-			{
-				m_pConnection->Disconnect();
-				bConnected = false;
-				Status = adFailed;
-			}
-			else
-			{
-				// freeing connection allows other threads to start.
-				// we doing this only if the problem was with article or group.
-				// if the problem occurs by connecting or authorization we do not
-				// free the connection, to prevent starting of thousands of threads 
-				// (cause each of them will also free it's connection after the 
-				// same connect-error).
-				FreeConnection(Status == adFinished || Status == adNotFound);
-			}
-		}
-
 		if (m_pConnection)
 		{
 			AddServerData();
 		}
 
-		if (Status == adFinished || Status == adFatalError)
+		if (!bConnected && m_pConnection)
 		{
-			break;
+			detail("Article %s @ %s failed: could not establish connection", m_szInfoName, m_szConnectionName);
 		}
 
-		pWantServer = NULL;
+		if (Status == adConnectError)
+		{
+			bConnected = false;
+			Status = adFailed;
+		}
 
 		if (bConnected && Status == adFailed)
 		{
 			iRemainedRetries--;
 		}
 
-		if (!bConnected || (Status == adFailed && iRemainedRetries > 0))
+		if (!bConnected && m_pConnection && !IsStopped())
+		{
+			g_pServerPool->BlockServer(pLastServer);
+		}
+
+		pWantServer = NULL;
+		if (bConnected && Status == adFailed && iRemainedRetries > 0 && !bRetentionFailure)
 		{
 			pWantServer = pLastServer;
 		}
-
-		if (pWantServer && 
-			!(IsStopped() || (g_pOptions->GetPauseDownload() && !bForce) ||
-			  (g_pOptions->GetTempPauseDownload() && !m_pFileInfo->GetExtraPriority()) ||
-			  iServerConfigGeneration != g_pServerPool->GetGeneration()))
+		else
 		{
-			detail("Waiting %i sec to retry", g_pOptions->GetRetryInterval());
-			SetStatus(adWaiting);
-			int msec = 0;
-			while (!(IsStopped() || (g_pOptions->GetPauseDownload() && !bForce) ||
-					 (g_pOptions->GetTempPauseDownload() && !m_pFileInfo->GetExtraPriority()) ||
-					 iServerConfigGeneration != g_pServerPool->GetGeneration()) &&
-				  msec < g_pOptions->GetRetryInterval() * 1000)
-			{
-				usleep(100 * 1000);
-				msec += 100;
-			}
-			SetLastUpdateTimeNow();
-			SetStatus(adRunning);
+			FreeConnection(Status == adFinished || Status == adNotFound);
+		}
+
+		if (Status == adFinished || Status == adFatalError)
+		{
+			break;
 		}
 
 		if (IsStopped() || (g_pOptions->GetPauseDownload() && !bForce) ||
@@ -243,7 +240,7 @@ void ArticleDownloader::Run()
 			break;
 		}
 
-		if (!pWantServer)
+		if (!pWantServer && (bConnected || bRetentionFailure))
 		{
 			failedServers.push_back(pLastServer);
 
@@ -397,7 +394,8 @@ ArticleDownloader::EStatus ArticleDownloader::Download()
 
 		// Throttle the bandwidth
 		while (!IsStopped() && (g_pOptions->GetDownloadRate() > 0.0f) &&
-		        (g_pStatMeter->CalcCurrentDownloadSpeed() > g_pOptions->GetDownloadRate()))
+			(g_pStatMeter->CalcCurrentDownloadSpeed() > g_pOptions->GetDownloadRate() ||
+			g_pStatMeter->CalcMomentaryDownloadSpeed() > g_pOptions->GetDownloadRate()))
 		{
 			SetLastUpdateTimeNow();
 			usleep(10 * 1000);
