@@ -2,7 +2,7 @@
  *  This file is part of nzbget
  *
  *  Copyright (C) 2005 Bo Cordes Petersen <placebodk@sourceforge.net>
- *  Copyright (C) 2007-2014 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2015 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1111 $
- * $Date: 2014-08-28 22:29:51 +0200 (Thu, 28 Aug 2014) $
+ * $Revision: 1221 $
+ * $Date: 2015-02-26 15:57:38 -0500 (Thu, 26 Feb 2015) $
  *
  */
 
@@ -61,7 +61,7 @@ extern void Reload();
 const char* g_szMessageRequestNames[] =
     { "N/A", "Download", "Pause/Unpause", "List", "Set download rate", "Dump debug", 
 		"Edit queue", "Log", "Quit", "Reload", "Version", "Post-queue", "Write log", "Scan", 
-		"Pause/Unpause postprocessor", "History", "Download URL" };
+		"Pause/Unpause postprocessor", "History" };
 
 const unsigned int g_iMessageRequestSizes[] =
     { 0,
@@ -78,10 +78,118 @@ const unsigned int g_iMessageRequestSizes[] =
 		sizeof(SNZBPostQueueRequest),
 		sizeof(SNZBWriteLogRequest),
 		sizeof(SNZBScanRequest),
-		sizeof(SNZBHistoryRequest),
-		sizeof(SNZBDownloadUrlRequest)
+		sizeof(SNZBHistoryRequest)
     };
 
+
+
+class BinCommand
+{
+protected:
+	Connection*			m_pConnection;
+	SNZBRequestBase*	m_pMessageBase;
+
+	bool				ReceiveRequest(void* pBuffer, int iSize);
+	void				SendBoolResponse(bool bSuccess, const char* szText);
+
+public:
+	virtual				~BinCommand() {}
+	virtual void		Execute() = 0;
+	void				SetConnection(Connection* pConnection) { m_pConnection = pConnection; }
+	void				SetMessageBase(SNZBRequestBase*	pMessageBase) { m_pMessageBase = pMessageBase; }
+};
+
+class DownloadBinCommand: public BinCommand
+{
+public:
+	virtual void		Execute();
+};
+
+class ListBinCommand: public BinCommand
+{
+public:
+	virtual void		Execute();
+};
+
+class LogBinCommand: public BinCommand
+{
+public:
+	virtual void		Execute();
+};
+
+class PauseUnpauseBinCommand: public BinCommand
+{
+public:
+	virtual void		Execute();
+};
+
+class EditQueueBinCommand: public BinCommand
+{
+public:
+	virtual void		Execute();
+};
+
+class SetDownloadRateBinCommand: public BinCommand
+{
+public:
+	virtual void		Execute();
+};
+
+class DumpDebugBinCommand: public BinCommand
+{
+public:
+	virtual void		Execute();
+};
+
+class ShutdownBinCommand: public BinCommand
+{
+public:
+	virtual void		Execute();
+};
+
+class ReloadBinCommand: public BinCommand
+{
+public:
+	virtual void		Execute();
+};
+
+class VersionBinCommand: public BinCommand
+{
+public:
+	virtual void		Execute();
+};
+
+class PostQueueBinCommand: public BinCommand
+{
+public:
+	virtual void		Execute();
+};
+
+class WriteLogBinCommand: public BinCommand
+{
+public:
+	virtual void		Execute();
+};
+
+class ScanBinCommand: public BinCommand
+{
+public:
+	virtual void		Execute();
+};
+
+class HistoryBinCommand: public BinCommand
+{
+public:
+	virtual void		Execute();
+};
+
+class UrlQueueBinCommand: public BinCommand
+{
+public:
+	virtual void		Execute();
+};
+
+
 //*****************************************************************
 // BinProcessor
 
@@ -182,10 +290,6 @@ void BinRpcProcessor::Dispatch()
 			command = new HistoryBinCommand();
 			break;
 
-		case eRemoteRequestDownloadUrl:
-			command = new DownloadUrlBinCommand();
-			break;
-
 		default:
 			error("Received unsupported request %i", ntohl(m_MessageBase.m_iType));
 			break;
@@ -330,30 +434,59 @@ void DownloadBinCommand::Execute()
 	}
 
 	int iBufLen = ntohl(DownloadRequest.m_iTrailingDataLength);
-	char* pRecvBuffer = (char*)malloc(iBufLen);
+	char* szNZBContent = (char*)malloc(iBufLen);
 
-	if (!m_pConnection->Recv(pRecvBuffer, iBufLen))
+	if (!m_pConnection->Recv(szNZBContent, iBufLen))
 	{
 		error("invalid request");
-		free(pRecvBuffer);
+		free(szNZBContent);
 		return;
 	}
 	
 	int iPriority = ntohl(DownloadRequest.m_iPriority);
 	bool bAddPaused = ntohl(DownloadRequest.m_bAddPaused);
 	bool bAddTop = ntohl(DownloadRequest.m_bAddFirst);
+	int iDupeMode = ntohl(DownloadRequest.m_iDupeMode);
+	int iDupeScore = ntohl(DownloadRequest.m_iDupeScore);
 
-	bool bOK = g_pScanner->AddExternalFile(DownloadRequest.m_szFilename, DownloadRequest.m_szCategory,
-		iPriority, NULL, 0, dmScore, NULL, bAddTop, bAddPaused, NULL, NULL, pRecvBuffer, iBufLen, NULL) != Scanner::asFailed;
+	bool bOK = false;
+
+	if (!strncasecmp(szNZBContent, "http://", 6) || !strncasecmp(szNZBContent, "https://", 7))
+	{
+		// add url
+		NZBInfo* pNZBInfo = new NZBInfo();
+		pNZBInfo->SetKind(NZBInfo::nkUrl);
+		pNZBInfo->SetURL(szNZBContent);
+		pNZBInfo->SetFilename(DownloadRequest.m_szNZBFilename);
+		pNZBInfo->SetCategory(DownloadRequest.m_szCategory);
+		pNZBInfo->SetPriority(iPriority);
+		pNZBInfo->SetAddUrlPaused(bAddPaused);
+		pNZBInfo->SetDupeKey(DownloadRequest.m_szDupeKey);
+		pNZBInfo->SetDupeScore(iDupeScore);
+		pNZBInfo->SetDupeMode((EDupeMode)iDupeMode);
+
+		DownloadQueue* pDownloadQueue = DownloadQueue::Lock();
+		pDownloadQueue->GetQueue()->Add(pNZBInfo, bAddTop);
+		pDownloadQueue->Save();
+		DownloadQueue::Unlock();
+
+		bOK = true;
+	}
+	else
+	{
+		bOK = g_pScanner->AddExternalFile(DownloadRequest.m_szNZBFilename, DownloadRequest.m_szCategory, iPriority,
+			DownloadRequest.m_szDupeKey, iDupeScore, (EDupeMode)iDupeMode, NULL, bAddTop, bAddPaused,
+			NULL, NULL, szNZBContent, iBufLen, NULL) != Scanner::asFailed;
+	}
 
 	char tmp[1024];
 	snprintf(tmp, 1024, bOK ? "Collection %s added to queue" : "Download Request failed for %s",
-		Util::BaseFileName(DownloadRequest.m_szFilename));
+		Util::BaseFileName(DownloadRequest.m_szNZBFilename));
 	tmp[1024-1] = '\0';
 
 	SendBoolResponse(bOK, tmp);
 
-	free(pRecvBuffer);
+	free(szNZBContent);
 }
 
 void ListBinCommand::Execute()
@@ -637,7 +770,7 @@ void LogBinCommand::Execute()
 		return;
 	}
 
-	Log::Messages* pMessages = g_pLog->LockMessages();
+	MessageList* pMessages = g_pLog->LockMessages();
 
 	int iNrEntries = ntohl(LogRequest.m_iLines);
 	unsigned int iIDFrom = ntohl(LogRequest.m_iIDFrom);
@@ -972,6 +1105,8 @@ void HistoryBinCommand::Execute()
 		return;
 	}
 
+	bool bShowHidden = ntohl(HistoryRequest.m_bHidden);
+
 	SNZBHistoryResponse HistoryResponse;
 	memset(&HistoryResponse, 0, sizeof(HistoryResponse));
 	HistoryResponse.m_MessageBase.m_iSignature = htonl(NZBMESSAGE_SIGNATURE);
@@ -985,16 +1120,27 @@ void HistoryBinCommand::Execute()
 	DownloadQueue* pDownloadQueue = DownloadQueue::Lock();
 
 	// calculate required buffer size for nzbs
-	int iNrEntries = pDownloadQueue->GetHistory()->size();
+	int iNrEntries = 0;
+	for (HistoryList::iterator it = pDownloadQueue->GetHistory()->begin(); it != pDownloadQueue->GetHistory()->end(); it++)
+	{
+		HistoryInfo* pHistoryInfo = *it;
+		if (pHistoryInfo->GetKind() != HistoryInfo::hkDup || bShowHidden)
+		{
+			iNrEntries++;
+		}
+	}
 	bufsize += iNrEntries * sizeof(SNZBHistoryResponseEntry);
 	for (HistoryList::iterator it = pDownloadQueue->GetHistory()->begin(); it != pDownloadQueue->GetHistory()->end(); it++)
 	{
 		HistoryInfo* pHistoryInfo = *it;
-		char szNicename[1024];
-		pHistoryInfo->GetName(szNicename, sizeof(szNicename));
-		bufsize += strlen(szNicename) + 1;
-		// align struct to 4-bytes, needed by ARM-processor (and may be others)
-		bufsize += bufsize % 4 > 0 ? 4 - bufsize % 4 : 0;
+		if (pHistoryInfo->GetKind() != HistoryInfo::hkDup || bShowHidden)
+		{
+			char szNicename[1024];
+			pHistoryInfo->GetName(szNicename, sizeof(szNicename));
+			bufsize += strlen(szNicename) + 1;
+			// align struct to 4-bytes, needed by ARM-processor (and may be others)
+			bufsize += bufsize % 4 > 0 ? 4 - bufsize % 4 : 0;
+		}
 	}
 
 	buf = (char*) malloc(bufsize);
@@ -1004,41 +1150,52 @@ void HistoryBinCommand::Execute()
 	for (HistoryList::iterator it = pDownloadQueue->GetHistory()->begin(); it != pDownloadQueue->GetHistory()->end(); it++)
 	{
 		HistoryInfo* pHistoryInfo = *it;
-		SNZBHistoryResponseEntry* pListAnswer = (SNZBHistoryResponseEntry*) bufptr;
-		pListAnswer->m_iID					= htonl(pHistoryInfo->GetID());
-		pListAnswer->m_iKind				= htonl((int)pHistoryInfo->GetKind());
-		pListAnswer->m_tTime				= htonl((int)pHistoryInfo->GetTime());
-
-		char szNicename[1024];
-		pHistoryInfo->GetName(szNicename, sizeof(szNicename));
-		pListAnswer->m_iNicenameLen			= htonl(strlen(szNicename) + 1);
-
-		if (pHistoryInfo->GetKind() == HistoryInfo::hkNzb)
+		if (pHistoryInfo->GetKind() != HistoryInfo::hkDup || bShowHidden)
 		{
-			NZBInfo* pNZBInfo = pHistoryInfo->GetNZBInfo();
-			unsigned long iSizeHi, iSizeLo;
-			Util::SplitInt64(pNZBInfo->GetSize(), &iSizeHi, &iSizeLo);
-			pListAnswer->m_iSizeLo				= htonl(iSizeLo);
-			pListAnswer->m_iSizeHi				= htonl(iSizeHi);
-			pListAnswer->m_iFileCount			= htonl(pNZBInfo->GetFileCount());
-			pListAnswer->m_iParStatus			= htonl(pNZBInfo->GetParStatus());
-			pListAnswer->m_iScriptStatus		= htonl(pNZBInfo->GetScriptStatuses()->CalcTotalStatus());
-		}
-		else if (pHistoryInfo->GetKind() == HistoryInfo::hkUrl)
-		{
-			NZBInfo* pNZBInfo = pHistoryInfo->GetNZBInfo();
-			pListAnswer->m_iUrlStatus			= htonl(pNZBInfo->GetUrlStatus());
-		}
+			SNZBHistoryResponseEntry* pListAnswer = (SNZBHistoryResponseEntry*) bufptr;
+			pListAnswer->m_iID					= htonl(pHistoryInfo->GetID());
+			pListAnswer->m_iKind				= htonl((int)pHistoryInfo->GetKind());
+			pListAnswer->m_tTime				= htonl((int)pHistoryInfo->GetTime());
 
-		bufptr += sizeof(SNZBHistoryResponseEntry);
-		strcpy(bufptr, szNicename);
-		bufptr += ntohl(pListAnswer->m_iNicenameLen);
-		// align struct to 4-bytes, needed by ARM-processor (and may be others)
-		if ((size_t)bufptr % 4 > 0)
-		{
-			pListAnswer->m_iNicenameLen = htonl(ntohl(pListAnswer->m_iNicenameLen) + 4 - (size_t)bufptr % 4);
-			memset(bufptr, 0, 4 - (size_t)bufptr % 4); //suppress valgrind warning "uninitialized data"
-			bufptr += 4 - (size_t)bufptr % 4;
+			char szNicename[1024];
+			pHistoryInfo->GetName(szNicename, sizeof(szNicename));
+			pListAnswer->m_iNicenameLen			= htonl(strlen(szNicename) + 1);
+
+			if (pHistoryInfo->GetKind() == HistoryInfo::hkNzb)
+			{
+				NZBInfo* pNZBInfo = pHistoryInfo->GetNZBInfo();
+				unsigned long iSizeHi, iSizeLo;
+				Util::SplitInt64(pNZBInfo->GetSize(), &iSizeHi, &iSizeLo);
+				pListAnswer->m_iSizeLo				= htonl(iSizeLo);
+				pListAnswer->m_iSizeHi				= htonl(iSizeHi);
+				pListAnswer->m_iFileCount			= htonl(pNZBInfo->GetFileCount());
+				pListAnswer->m_iParStatus			= htonl(pNZBInfo->GetParStatus());
+				pListAnswer->m_iScriptStatus		= htonl(pNZBInfo->GetScriptStatuses()->CalcTotalStatus());
+			}
+			else if (pHistoryInfo->GetKind() == HistoryInfo::hkDup && bShowHidden)
+			{
+				DupInfo* pDupInfo = pHistoryInfo->GetDupInfo();
+				unsigned long iSizeHi, iSizeLo;
+				Util::SplitInt64(pDupInfo->GetSize(), &iSizeHi, &iSizeLo);
+				pListAnswer->m_iSizeLo				= htonl(iSizeLo);
+				pListAnswer->m_iSizeHi				= htonl(iSizeHi);
+			}
+			else if (pHistoryInfo->GetKind() == HistoryInfo::hkUrl)
+			{
+				NZBInfo* pNZBInfo = pHistoryInfo->GetNZBInfo();
+				pListAnswer->m_iUrlStatus			= htonl(pNZBInfo->GetUrlStatus());
+			}
+
+			bufptr += sizeof(SNZBHistoryResponseEntry);
+			strcpy(bufptr, szNicename);
+			bufptr += ntohl(pListAnswer->m_iNicenameLen);
+			// align struct to 4-bytes, needed by ARM-processor (and may be others)
+			if ((size_t)bufptr % 4 > 0)
+			{
+				pListAnswer->m_iNicenameLen = htonl(ntohl(pListAnswer->m_iNicenameLen) + 4 - (size_t)bufptr % 4);
+				memset(bufptr, 0, 4 - (size_t)bufptr % 4); //suppress valgrind warning "uninitialized data"
+				bufptr += 4 - (size_t)bufptr % 4;
+			}
 		}
 	}
 
@@ -1058,42 +1215,3 @@ void HistoryBinCommand::Execute()
 
 	free(buf);
 }
-
-void DownloadUrlBinCommand::Execute()
-{
-	SNZBDownloadUrlRequest DownloadUrlRequest;
-	if (!ReceiveRequest(&DownloadUrlRequest, sizeof(DownloadUrlRequest)))
-	{
-		return;
-	}
-
-	URL url(DownloadUrlRequest.m_szURL);
-	if (!url.IsValid())
-	{
-		char tmp[1024];
-		snprintf(tmp, 1024, "Url %s is not valid", DownloadUrlRequest.m_szURL);
-		tmp[1024-1] = '\0';
-		SendBoolResponse(true, tmp);
-		return;
-	}
-
-	NZBInfo* pNZBInfo = new NZBInfo();
-	pNZBInfo->SetKind(NZBInfo::nkUrl);
-	pNZBInfo->SetURL(DownloadUrlRequest.m_szURL);
-	pNZBInfo->SetFilename(DownloadUrlRequest.m_szNZBFilename);
-	pNZBInfo->SetCategory(DownloadUrlRequest.m_szCategory);
-	pNZBInfo->SetPriority(ntohl(DownloadUrlRequest.m_iPriority));
-	pNZBInfo->SetAddUrlPaused(ntohl(DownloadUrlRequest.m_bAddPaused));
-
-	DownloadQueue* pDownloadQueue = DownloadQueue::Lock();
-	pDownloadQueue->GetQueue()->Add(pNZBInfo, ntohl(DownloadUrlRequest.m_bAddFirst));
-	pDownloadQueue->Save();
-	DownloadQueue::Unlock();
-
-	info("Request: Queue url %s", DownloadUrlRequest.m_szURL);
-
-	char tmp[1024];
-	snprintf(tmp, 1024, "Url %s added to queue", DownloadUrlRequest.m_szURL);
-	tmp[1024-1] = '\0';
-	SendBoolResponse(true, tmp);
-}
