@@ -1,7 +1,7 @@
 /*
  *  This file is part of nzbget
  *
- *  Copyright (C) 2007-2014 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2015 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1136 $
- * $Date: 2014-10-04 21:34:03 +0200 (Sat, 04 Oct 2014) $
+ * $Revision: 1271 $
+ * $Date: 2015-04-29 23:08:54 +0200 (Wed, 29 Apr 2015) $
  *
  */
 
@@ -77,6 +77,7 @@ public:
 						~StringBuilder();
 	void				Append(const char* szStr);
 	const char*			GetBuffer() { return m_szBuffer; }
+	void				Clear();
 };
 
 class Util
@@ -109,10 +110,12 @@ public:
 	static void FixExecPermission(const char* szFilename);
 #endif
 	static void ExpandFileName(const char* szFilename, char* szBuffer, int iBufSize);
+	static void GetExeFileName(const char* argv0, char* szBuffer, int iBufSize);
 	static void FormatFileSize(char* szBuffer, int iBufLen, long long lFileSize);
 	static bool SameFilename(const char* szFilename1, const char* szFilename2);
 	static bool MatchFileExt(const char* szFilename, const char* szExtensionList, const char* szListSeparator);
 	static char* GetLastErrorMessage(char* szBuffer, int iBufLen);
+	static void FormatSpeed(int iBytesPerSecond, char* szBuffer, int iBufSize);
 
 	/*
 	 * Split command line int arguments.
@@ -235,6 +238,12 @@ public:
 	 */
 	static void HttpUnquote(char* raw);
 
+	/*
+	 * Decodes URL-string.
+	 * The string is decoded on the place overwriting the content of raw-data.
+	 */
+	static void URLDecode(char* raw);
+
 #ifdef WIN32
 	static bool Utf8ToAnsi(char* szBuffer, int iBufLen);
 	static bool AnsiToUtf8(char* szBuffer, int iBufLen);
