@@ -1,7 +1,7 @@
 /*
  *  This file is part of nzbget
  *
- *  Copyright (C) 2007-2014 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2015 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1175 $
- * $Date: 2014-11-26 23:16:23 +0100 (Wed, 26 Nov 2014) $
+ * $Revision: 1288 $
+ * $Date: 2015-05-19 21:25:35 +0200 (Tue, 19 May 2015) $
  *
  */
 
@@ -71,7 +71,7 @@
 /* Define to 1 if spinlocks are supported */
 #define HAVE_SPINLOCK
 
-#define VERSION "14.1"
+#define VERSION "15.0"
 
 /* Suppress warnings */
 #define _CRT_SECURE_NO_DEPRECATE
@@ -81,6 +81,11 @@
 
 #define _USE_32BIT_TIME_T
 
+#if _WIN32_WINNT < 0x0501
+#undef _WIN32_WINNT
+#define _WIN32_WINNT   0x0501
+#endif
+
 #ifdef _DEBUG
 // detection of memory leaks
 #define _CRTDBG_MAP_ALLOC
